// MISC.CPP
// Miscellaneous functions.
// Copyright (c) 2001 - 2005, Carson F. Ball
// Originally from the project Mailer.
// 09/11/2002 -

#include <ctype.h>  // Needed for isalpha.
#include <stdio.h>  // Needed to printf.
#include <string.h> // Needed for strupr.

#include "mtypes.h"
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  IsNum                                                         |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to check.                               |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    True if the string is a number, false otherwise.              |
// +-------------------------------------------------------------------------+
// |char       Value    - The value to return.                               |
// +-------------------------------------------------------------------------+
// |This function checks to see if a string contains only decimal digits and |
// |periods.                                                                 |
// +-------------------------------------------------------------------------+
bool IsNum (const char* String) {
   char Value = true;

   while (*String && Value) {
      if (!(*String == ' ')) {
         if (!isdigit (*String) && *String != '.') {
            Value = false;
         }
      }

      ++String;
   }

   return (Value);
}

#ifdef __linux__
// +-------------------------------------------------------------------------+
// |FUNCTION:  strupr                                                        |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to change to upper case.                |
// |OUTPUT:    String   - The string that has been converted to upper case.  |
// |RETURN:    The string that has been converted to upper case.             |
// +-------------------------------------------------------------------------+
// |int        Length   - The length of the string to convert.               |
// +-------------------------------------------------------------------------+
// |This function converts a string to upper case.                           |
// +-------------------------------------------------------------------------+
char* strupr (char* String) {
   int Length = strlen (String);

   for (int Count = 0; Count < Length; ++Count) {
      if (isalpha (String [Count])) {
         String [Count] &= 0xDF;
      }
   }

   return (String);
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  strcmpi                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     String1  - The first string to compare.                       |
// |OUTPUT:    String2  - The second string to compare.                      |
// |RETURN:    0 if the strings are the same.                                |
// +-------------------------------------------------------------------------+
// |char*      Str1     - A pointer to the first string.                     |
// |char*      Str2     - A pointer to the second string.                    |
// +-------------------------------------------------------------------------+
// |This function compares two strings without regard to case. If the        |
// |strings are identical, 0 is returned.  If String1 is less than String2,  |
// |a negative number is returned.  Otherwise, a positive number is          |
// |returned.                                                                |
// +-------------------------------------------------------------------------+
int strcmpi (const char* String1, const char* String2) {
   const char* Str1 = String1;
   const char* Str2 = String2;

   while (*Str1 && ((*Str1 == *Str2) || ((*Str1 & 0xDF) == (*Str2 & 0xDF)))) {
      ++Str1;
      ++Str2;
  }

  return ((*Str1 & 0xDF) - (*Str2 & 0xDF));
}

#endif
