// SOCK.CPP
// SMTP access routines.
// Copyright (c) 2000 - 2005, Carson F. Ball`
// Originally from the project Mailer.
// 12/05/2000 -

#define SOCK

#include <stdio.h>  // Needed for printf.
#include <string.h> // Needed for strlen.

#include "sock.h"
#include "misc.h"

// +-------------------------------------------------------------------------+
// |FUNCTION:  CONNECT                                                       |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function is the constructor for the CONNECT class.                  |
// +-------------------------------------------------------------------------+
CONNECT::CONNECT (void) {
   Status = 0;

#ifdef __WIN32__
   WSAData WData;

   if (WSAStartup (MAKEWORD (1, 1), &WData) != 0) {
      // Winsock could not be started.
      Status = 12;
   }
   else
#endif
   if ((Sock = socket (PF_INET, SOCK_STREAM, 0)) < 0) {
      // The socket could not be created.
      Status = 13;
   }

   return;
}

CONNECT::CONNECT (const char* Address, UINT16 Port) {
   Status = 0;

#ifdef __WIN32__
   WSADATA WData;

   if (WSAStartup (MAKEWORD (1, 1), &WData) != 0) {
      // Winsock could not be started.
      Status = 12;
   }
   else
#endif
   if ((Sock = socket (PF_INET, SOCK_STREAM, 0)) < 0) {
      // The socket could not be created.
      Status = 13;

      return;
   }

//   memset (&Server, 0, sizeof (hostent));
   
   if (IsNum (Address)) {
      Server.sin_addr.s_addr = inet_addr (Address);
   }
   else {
      Host = gethostbyname (Address);
      if (!Host) {
         // The host could not be located.
         Status = 3;

         return;
      }
      else {
         memcpy (&Server.sin_addr.s_addr,
                 Host->h_addr,
                 sizeof (Server.sin_addr.s_addr));
      }
   }

   Server.sin_family      = PF_INET;
   Server.sin_port        = htons (Port);

   if ((connect (Sock, (sockaddr*) &Server, sizeof (Server))) == -1) {
      // Could not connect to the server.
      Status = 4;
   }

   return;
}
// +-------------------------------------------------------------------------+
// |FUNCTION:  ~CONNECT                                                      |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function is the destructor for the CONNECT class.                   |
// +-------------------------------------------------------------------------+
CONNECT::~CONNECT (void) {
#ifdef __WIN32__
   closesocket (Sock);
   WSACleanup  ();
#elif __linux__
   close (Sock);
#endif

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Send                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     String   - The string to send to the server.                  |
// |           Size     - The size of the string (0 by default).             |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function sends a string to the mail server.  If no size is          |
// |specified, the size is calculated.                                       |
// +-------------------------------------------------------------------------+
void CONNECT::Send (const char* String, UINT32 Size) {
   if (!Size) {
      Size = strlen (String);
   }

   if (send (Sock, String, Size, MSG_DONTROUTE) < 0) {
      Status = 6;
   }

   return;
}

// +-------------------------------------------------------------------------+
// |FUNCTION:  Recv                                                          |
// +-------------------------------------------------------------------------+
// |INPUT:     Nothing.                                                      |
// |OUTPUT:    Nothing.                                                      |
// |RETURN:    Nothing.                                                      |
// +-------------------------------------------------------------------------+
// |This function receives a string from the mail server and stores it in a  |
// |buffer.                                                                  |
// +-------------------------------------------------------------------------+
void CONNECT::Recv (void) {
   memset (Buffer, 0, BUFLEN);

   // 0 removes the information from the server after reading it.
   if (recv (Sock, Buffer, BUFLEN, 0) < 0) {
      Status = 5;
   }

   return;
}

const char* CONNECT::GetBuffer (void) {
   return (Buffer);
}
